<?php
/**
* @author      Opensource Design, http://www.opensourcedesign.nl
* @copyright   Copyright (C) 2014 Opensource Design. All rights reserved.
* @license     GNU General Public License, http://www.gnu.org/copyleft/gpl.html
* @Version     1.0.0
**/

// no direct access
defined('_JEXEC') or die;

jimport('joomla.plugin.plugin');

class plgContentF2cEditArticleButton extends JPlugin
{
	public function onContentPrepare($context, &$row, &$params, $page = 0)
	{
		// Don't run this plugin when the content is being indexed
		if ($context == 'com_finder.indexer')
		{
			return true;
		}
		
		if (JString::strpos($row->text, '{plgContentF2cEditArticleButton}') === false) 
		{
			return true;
		}

		JPlugin::loadLanguage('plg_content_f2ceditarticlebutton');
		
		$buttonHtml = '';
		
		//Find all instances of plugin and put in $matches
		preg_match_all('/{plgContentF2cEditArticleButton}{\/plgContentF2cEditArticleButton}/iU', $row->text, $matches);

		for ($i=0; $i < count($matches[0]); $i++)
		{
			// Test if this is a F2C Article
			$form = $this->getForm($row->id);
			
			if($form)
			{
				if($this->canEdit($form))
				{					
					// Create the edit url
					$url = 'index.php?option=com_form2content&task=form.edit&id='.$form->id;
					
					switch($this->params->get('append_itemid', 0))
					{
						case 0:
							// Do not append Itemid
							break;							
						case 1:
							// Append the current Itemid of the currently displayed page
							$Jinput = JFactory::getApplication()->input;
							$url .= '&Itemid='.$Jinput->getInt('Itemid');
							break;
						case 2:
							// Calculate the Itemid by evaluating the user rules. The first match will determine the Itemid
							$itemId = $this->evaluateRules($form->projectid);

							if($itemId)
							{
								$url .= '&Itemid='.$itemId;
							}
							break;
					}
					
					$url .= '&return='.base64_encode(JUri::getInstance());
					
					// Create the html for the button/link			
					$buttonHtml = $this->params->get('button_html');
					
					if(empty($buttonHtml))
					{
						$buttonHtml = '<a href="{$LINK}">' . JText::_('PLG_F2CEDITARTICLEBUTTON_EDITARTICLE') . '</a>';	
					}
					
					$buttonHtml = str_ireplace('{$LINK}', JRoute::_($url), $buttonHtml);
				}
			}
						
			$row->text = str_replace($matches[0][$i], $buttonHtml, $row->text);			
		}

		return true;
	}
	
	private function canEdit($form)
	{
		$user	= JFactory::getUser();
		$asset	= 'com_form2content.form.'.$form->id;

		// Check general edit permission first.
		if ($user->authorise('core.edit', $asset)) 
		{ 
			return true;
		}

		// Fallback on edit.own.
		// First test if the permission is available.
		if ($user->authorise('core.edit.own', $asset) && $form->created_by == $user->get('id')) 
		{
			return true;
		}
		
		return false;
	}
	
	private function getForm($id)
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$query->select('*');
		$query->from('#__f2c_form');
		$query->where('reference_id = ' . (int)$id);
			
		$db->setQuery($query);
		
		return $db->loadObject();		
	}

	private function evaluateRules($contentTypeId)
	{
		$rules 	= explode(PHP_EOL, $this->params->get('rules_itemid'));
		
		foreach($rules as $rule)
		{
			$arrRules 	= array();
			$ruleParts 	= explode(';', $rule);
			
			foreach($ruleParts as $rulePart)
			{
				$arrRuleItem = explode('=', $rulePart);
				
				if(count($arrRuleItem) != 2)
				{
					// invalid rule item, syntax is not equal to key=value
					continue;
				}
				
				$arrRules[$arrRuleItem[0]] = $arrRuleItem[1];
			}
			
			$Itemid = $this->evaluateRule($arrRules, $contentTypeId);
			
			if(!empty($Itemid))
			{
				// We have a match!
				return $Itemid;
			}
		}
		
		// No matching rules were found
		return null;
	}
	
	private function evaluateRule($ruleParts, $contentTypeId)
	{
		if(!array_key_exists('itemid', $ruleParts))
		{
			// Invalid rule
			return null;
		}
		
		if(array_key_exists('lang', $ruleParts))
		{
			// Get the language as used on the sute
			$lang 			= JFactory::getLanguage();
			$arrlanguageTag = explode('-', $lang->getTag());
			
			// Is it a full language tag like en-GB or a partial one like en?
			$languageParts = explode('-', $ruleParts['lang']);
			
			if(count($languageParts) == 1)
			{
				if($arrlanguageTag[0] != $languageParts[0])
				{
					// no matching ContentType
					return null;
				}
			}
			else
			{
				if($arrlanguageTag[0] != $languageParts[0] || $arrlanguageTag[1] != $languageParts[1])
				{
					// no matching ContentType
					return null;
				}
			}
		}
		
		if(array_key_exists('contenttype', $ruleParts) && $ruleParts['contenttype'] != $contentTypeId)
		{
			// no matching ContentType
			return null;
		}
		
		return $ruleParts['itemid'];
	}
}
